﻿using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Windows.Data;
using System.Windows.Input;

namespace WpfApplication2
{
    public class MainWindowViewModel : INotifyPropertyChanged
    {
        public event PropertyChangedEventHandler PropertyChanged;

        private ObservableCollection<Foo> _foos;
        private ICollectionView _foosView;
        private ICommand _loadCommand;
        private ICommand _previousCommand;
        private ICommand _nextCommand;
        private ICommand _removeCommand;

        public ObservableCollection<Foo> Foos
        {
            get
            {
                if (_foos == null)
                {
                    _foos = new ObservableCollection<Foo>();
                    _foosView = CollectionViewSource.GetDefaultView(_foos);
                    LoadCommand.Execute(null);
                }

                return _foos;
            }
        }

        public ICollectionView FoosView
        {
            get { return _foosView; }
        }

        public ICommand LoadCommand
        {
            get
            {
                if (_loadCommand == null)
                {
                    _loadCommand = new RelayCommand(InitializeFoos);
                }

                return _loadCommand;
            }
        }

        public ICommand PreviousCommand
        {
            get
            {
                if (_previousCommand == null)
                {
                    _previousCommand = new RelayCommand(Previous);
                }

                return _previousCommand;
            }
        }

        public ICommand NextCommand
        {
            get
            {
                if (_nextCommand == null)
                {
                    _nextCommand = new RelayCommand(Next);
                }

                return _nextCommand;
            }
        }

        public ICommand RemoveCommand
        {
            get
            {
                if (_removeCommand == null)
                {
                    _removeCommand = new RelayCommand(Remove);
                }

                return _removeCommand;
            }
        }

        public virtual void OnPropertyChanged(string propertyName)
        {
            PropertyChangedEventHandler handler = PropertyChanged;
            if (handler != null)
            {
                handler(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        private void InitializeFoos(object parameter)
        {
            Foos.Clear();

            for (int i = 1; i <= 9; i++)
            {
                Foos.Add(new Foo { Identifier = i, FirstName = "FirstName " + i, LastName = "LastName " + i });
            }

            // Because the Infragistics grid is bound directly to the ListCollectionView on start,
            // we need to trigger a PropertyChanged event.
            OnPropertyChanged("FoosView");
        }

        private void Previous(object parameter)
        {
            _foosView.MoveCurrentToPrevious();
        }

        private void Next(object parameter)
        {
            _foosView.MoveCurrentToNext();
        }

        private void Remove(object parameter)
        {
            _foos.Remove(_foosView.CurrentItem as Foo);
        }
    }
}
